@@ -8,17 +8,18 @@
 #
 
 
-setGeneric("to_grofit_time",
-  function(object, ...) standardGeneric("to_grofit_time"))
 #' Times for grofit
 #'
 #' Construct time-points data frame as required by \code{grofit}.
 #'
 #' @param object \code{\link{OPM}} object.
-#' @return Dataframe with time points in each row, repeated for each well
+#' @return Data frame with time points in each row, repeated for each well
 #'   (number of rows is number of wells).
 #' @keywords internal
 #'
+setGeneric("to_grofit_time",
+  function(object, ...) standardGeneric("to_grofit_time"))
+
 setMethod("to_grofit_time", OPM, function(object) {
   tp <- hours(object, "all")
   as.data.frame(matrix(rep.int(tp, length(wells(object))), ncol = length(tp),
@@ -29,17 +30,18 @@ setMethod("to_grofit_time", OPM, function(object) {
 ################################################################################
 
 
-setGeneric("to_grofit_data",
-  function(object, ...) standardGeneric("to_grofit_data"))
 #' Data for grofit
 #'
 #' Construct data frame with measurements as required by \code{grofit}.
 #'
 #' @param object \code{\link{OPM}} object.
-#' @return Dataframe with columns: (i) well ID, (ii) plate ID, (iii) dummy
+#' @return Data frame with columns: (i) well ID, (ii) plate ID, (iii) dummy
 #'   concentration, (iv - end) measurements, one row for each well.
 #' @keywords internal
 #'
+setGeneric("to_grofit_data",
+  function(object, ...) standardGeneric("to_grofit_data"))
+
 setMethod("to_grofit_data", OPM, function(object) {
   w <- wells(object)
   names <- matrix(nrow = length(w), ncol = 3L,
@@ -57,24 +59,25 @@ setMethod("to_grofit_data", OPM, function(object) {
 
 ## NOTE: Not an S4 method because 'grofit' is an S3 class
 
-extract_curve_params <- function(data) UseMethod("extract_curve_params")
 #' Grofit extraction
 #'
 #' Extract and rename estimated curve parameters.
 #'
-#' @param data Object of class \sQuote{grofit}.
+#' @param x Object of class \sQuote{grofit}.
 #' @return Matrix.
-#'
-#' @rdname extract_curve_params
-#' @method extract_curve_params grofit
 #' @keywords internal
 #'
-extract_curve_params.grofit <- function(data) {
-  settings <- c(data$control)
-  data <- summary(data$gcFit)
+extract_curve_params <- function(x) UseMethod("extract_curve_params")
+
+#' @rdname extract_curve_params
+#' @method extract_curve_params grofit
+#'
+extract_curve_params.grofit <- function(x) {
+  settings <- c(x$control)
+  x <- summary(x$gcFit)
   map <- map_grofit_names()
-  structure(.Data = t(as.matrix(data[, names(map)])),
-    dimnames = list(map, data[, "TestId"]), settings = settings)
+  structure(.Data = t(as.matrix(x[, names(map)])),
+    dimnames = list(map, x[, "TestId"]), settings = settings)
 }
 
 
@@ -94,8 +97,7 @@ extract_curve_params.grofit <- function(data) {
 #' @keywords utilities
 #' @examples
 #' (x <- param_names())
-#' stopifnot(is.character(x), length(x) == 4L)
-#' stopifnot(identical(unique(x), x))
+#' stopifnot(is.character(x), length(x) == 4L, identical(unique(x), x))
 #'
 param_names <- function() {
   CURVE_PARAMS
@@ -108,26 +110,26 @@ param_names <- function() {
 #' Aggregate kinetics using curve-parameter estimation
 #'
 #' Aggregate the kinetic data using curve-parameter estimation, i.e. infer
-#' parameters from the kinetic data stored in an \code{\link{OPM}}
-#' object using either the \pkg{grofit} package or the built-in method.
-#' Optionally include the aggregated values in a novel \code{\link{OPMA}}
-#' object together with previously collected information.
+#' parameters from the kinetic data stored in an \code{\link{OPM}} object using
+#' either the \pkg{grofit} package or the built-in method. Optionally include
+#' the aggregated values in a novel \code{\link{OPMA}} object together with
+#' previously collected information.
 #'
 #' @param object \code{\link{OPM}} or \code{\link{OPMS}} object. There is also
 #'   a helper method for matrix objects.
 #' @param boot Integer scalar. Number of bootstrap replicates used to estimate
-#'   95-percent confidence intervals (CIs) for the parameter. Set this to zero
+#'   95-percent confidence intervals (CIs) for the parameters. Set this to zero
 #'   to omit bootstrapping, resulting in \code{NA} entries for the CIs.
 #' @param verbose Logical scalar. Print progress messages?
-#' @param cores Integer scalar. Number of cores to use. Setting this to a
-#'   value > 1 requires the \pkg{multicore} package. Has no effect if
+#' @param cores Integer scalar. Number of cores to use. Setting this to a value
+#'   > 1 requires the \pkg{multicore} package. Has no effect if
 #'   \sQuote{opm-fast} is chosen (see below).
 #' @param options List. For its use in \sQuote{grofit} mode, see
 #'   \code{grofit.control} in the \pkg{grofit} package. The \code{boot} and
 #'   \code{verbose} settings, as the most important ones, are added separately
 #'   (see above). The verbose mode is not very useful in parallel processing.
 #'   For its use in \sQuote{opm-fast} mode, see \code{\link{fast_estimate}}.
-#' @param program Character scalar. The aggregation method to use. Currently
+#' @param method Character scalar. The aggregation method to use. Currently
 #'   only the following methods are supported:
 #'   \describe{
 #'     \item{grofit}{The \code{grofit} function in the eponymous package, with
@@ -140,6 +142,9 @@ param_names <- function() {
 #'     however, the median bootstrap value is preferred as point estimate over
 #'     the real point estimate.}
 #'   }
+#' @param program Deprecated. Use \sQuote{method} instead. If provided,
+#'   \sQuote{program} has precedence over \sQuote{method}, but \sQuote{program}
+#'   will eventually be removed.
 #' @param plain Logical scalar. If \code{TRUE}, only the aggregated values are
 #'   returned (as a matrix, for details see below). Otherwise they are
 #'   integrated in an \code{\link{OPMA}} object together with \code{object}.
@@ -156,23 +161,33 @@ param_names <- function() {
 #' @export
 #' @return If \code{plain} is \code{FALSE}, an \code{\link{OPMA}} object.
 #'   Otherwise a numeric matrix of the same structure than the one returned by
-#'   \code{\link{aggregated}} but with an additional \sQuote{settings}
-#'   attribute containing the (potentially modified) list proved via the
-#'   \code{settings} argument, and a \sQuote{program} attribute corresponding
-#'   to the \code{program} argument. The matrix method returns a matrix.
+#'   \code{\link{aggregated}} but with an additional \sQuote{settings} attribute
+#'   containing the (potentially modified) list proved via the \code{settings}
+#'   argument, and a \sQuote{method} attribute corresponding to the
+#'   \code{method} argument. The matrix method returns a matrix.
 #'
 #' @family aggregation-functions
 #' @seealso grofit::grofit
 #' @keywords smooth
 #'
+#' @details Behaviour is special if the \code{\link{plate_type}} is one of those
+#'   that have to be explicitly set using \code{\link{gen_iii}} and there is
+#'   just one point measurement. Because this behaviour is usual for plates
+#'   measured either in Generation-III (identification) mode or on a
+#'   MicroStation\eqn{\textsuperscript{\texttrademark}}{(TM)}, the point
+#'   estimate is simply regarded as \sQuote{A} parameter (maximum height) and
+#'   all other parameters are set to \code{NA}.
+#'
 #' @note \itemize{
+#'   \item The aggregated values can be queried for using \code{\link{has_aggr}}
+#'     and received using \code{\link{aggregated}}.
 #'   \item The \sQuote{OPMS} method just applies the \sQuote{OPM} method to
-#'     each contained plate in turn; there are not interdependencies.
+#'     each contained plate in turn; there are not inter-dependencies.
 #'   \item Examples with \code{plain = TRUE} are not given, as only the return
 #'     value is different: Let \code{x} be the normal result of
 #'     \code{do_aggr()}. The matrix returned if \code{plain} is \code{TRUE}
 #'     could then be received using \code{aggregated(x)}, whereas the
-#'     \sQuote{program} and the \sQuote{settings} attributes could be obtained
+#'     \sQuote{method} and the \sQuote{settings} attributes could be obtained
 #'     as components of the list returned by \code{aggr_settings(x)}.
 #' }
 #'
@@ -195,19 +210,21 @@ param_names <- function() {
 #' data(vaas_1)
 #'
 #' # Run a fast estimate of A and AUC without bootstrapping
-#' copy <- do_aggr(vaas_1, program = "opm-fast", boot = 0,
+#' copy <- do_aggr(vaas_1, method = "opm-fast", boot = 0,
 #'   options = list(as.pe = "pe"))
+#' aggr_settings(vaas_1)
+#' aggr_settings(copy)
 #' stopifnot(has_aggr(vaas_1), has_aggr(copy))
-#' stopifnot(identical(aggr_settings(vaas_1)$program, "grofit"))
-#' stopifnot(identical(aggr_settings(copy)$program, "opm-fast"))
 #'
 #' # Compare the results to the ones precomputed with grofit
+#' # (1) A
 #' a.grofit <- aggregated(vaas_1, "A", ci = FALSE)
 #' a.fast <-  aggregated(copy, "A", ci = FALSE)
 #' plot(a.grofit, a.fast)
 #' stopifnot(cor.test(a.fast, a.grofit)$estimate > 0.999)
+#' # (2) AUC
 #' auc.grofit <- aggregated(vaas_1, "AUC", ci = FALSE)
-#' auc.fast <-  aggregated(copy, "AUC", ci = FALSE)
+#' auc.fast <- aggregated(copy, "AUC", ci = FALSE)
 #' plot(auc.grofit, auc.fast)
 #' stopifnot(cor.test(auc.fast, auc.grofit)$estimate > 0.999)
 #'
@@ -234,14 +251,15 @@ param_names <- function() {
 setGeneric("do_aggr", function(object, ...) standardGeneric("do_aggr"))
 
 setMethod("do_aggr", OPM, function(object, boot = 100L, verbose = FALSE,
-    cores = 1L, options = list(), program = "grofit", plain = FALSE) {
+    cores = 1L, options = list(), method = "grofit", plain = FALSE,
+    program = method) {
 
   # Convert to OPMA
   integrate_in_opma <- function(object, result) {
-    settings <- list(program = attr(result, PROGRAM),
-      options = attr(result, OPTIONS))
-    attr(result, PROGRAM) <- NULL
-    attr(result, OPTIONS) <- NULL
+    items <- c(METHOD, OPTIONS, SOFTWARE, VERSION)
+    settings <- attributes(result)[items]
+    for (item in items)
+      attr(result, item) <- NULL
     new(OPMA, measurements = measurements(object),
       metadata = metadata(object), csv_data = csv_data(object),
       aggregated = result, aggr_settings = settings)
@@ -249,7 +267,7 @@ setMethod("do_aggr", OPM, function(object, boot = 100L, verbose = FALSE,
 
   # Add our own changes of the default
   make_grofit_control <- function(verbose, boot, add) {
-    result <- grofit::grofit.control()
+    result <- grofit.control()
     orig.class <- class(result)
     result <- insert(unclass(result), interactive = FALSE,
       suppress.messages = !verbose, fit.opt = "s", nboot.gc = boot,
@@ -260,47 +278,65 @@ setMethod("do_aggr", OPM, function(object, boot = 100L, verbose = FALSE,
   }
 
   run_grofit <- function(time, data, control) {
-    result <- grofit::grofit(time = time, data = data, ec50 = FALSE,
-      control = control)
-    extract_curve_params(result)
+    extract_curve_params(grofit(time = time, data = data,
+      ec50 = FALSE, control = control))
   }
 
-  case(program <- match.arg(program, KNOWN_PROGRAMS),
+  copy_A_param <- function(x) {
+    map <- unlist(map_grofit_names(opm.fast = TRUE))
+    result <- matrix(data = NA_real_, nrow = length(map), ncol = length(x),
+      dimnames = list(unname(map), names(x)))
+    result[map[["A.point.est"]], ] <- x
+    result
+  }
 
-    grofit = {
-      control <- make_grofit_control(verbose, boot, add = options)
-      grofit.time <- to_grofit_time(object)
-      grofit.data <- to_grofit_data(object)
-      result <- traverse(as.list(seq.int(nrow(grofit.data))),
-        fun = function(row) {
-          run_grofit(grofit.time[row, , drop = FALSE],
-          grofit.data[row, , drop = FALSE], control)
-        }, cores = cores)
-      result <- do.call(cbind, result)
-      attr(result, OPTIONS) <- unclass(control)
-    },
+  if (!missing(program)) {
+    warning("'program' is deprecated -- use 'method' instead")
+    method <- program
+  }
 
-    `opm-fast` = {
-      options <- insert(as.list(options), boot = boot, .force = FALSE)
-      mat <- measurements(object)
-      result <- rbind(
-        do.call(fast_estimate, c(list(x = mat, what = "AUC"), options)),
-        do.call(fast_estimate, c(list(x = mat, what = "A"), options)),
-        matrix(nrow = 6L, ncol = ncol(mat) - 1L, data = NA_real_)
-      )
-      rownames(result)[7L:9L] <- sub("^[^.]+", "lambda",
-        rownames(result)[1L:3L])
-      rownames(result)[10L:12L] <- sub("^[^.]+", "mu",
-        rownames(result)[1L:3L])
-      map <- map_grofit_names(opm.fast = TRUE)
-      result <- result[names(map), , drop = FALSE]
-      rownames(result) <- as.character(map)
-      attr(result, OPTIONS) <- options
-    }
+  if (plate_type(object) %in% SPECIAL_PLATES && dim(object)[1] < 2L) {
+    result <- copy_A_param(well(object))
+    attr(result, METHOD) <- "shortcut"
+    attr(result, OPTIONS) <- list(boot = boot)
+  } else {
+    case(method <- match.arg(method, KNOWN_METHODS$aggregation),
+      grofit = {
+        control <- make_grofit_control(verbose, boot, add = options)
+        grofit.time <- to_grofit_time(object)
+        grofit.data <- to_grofit_data(object)
+        result <- traverse(as.list(seq.int(nrow(grofit.data))),
+          fun = function(row) {
+            run_grofit(grofit.time[row, , drop = FALSE],
+              grofit.data[row, , drop = FALSE], control)
+          }, cores = cores)
+          result <- do.call(cbind, result)
+          attr(result, OPTIONS) <- unclass(control)
+      },
+      `opm-fast` = {
+        options <- insert(as.list(options), boot = boot, .force = FALSE)
+        mat <- measurements(object)
+        result <- rbind(
+          do.call(fast_estimate, c(list(x = mat, what = "AUC"), options)),
+          do.call(fast_estimate, c(list(x = mat, what = "A"), options)),
+          matrix(nrow = 6L, ncol = ncol(mat) - 1L, data = NA_real_)
+        )
+        rownames(result)[7L:9L] <- sub("^[^.]+", "lambda",
+          rownames(result)[1L:3L], perl = TRUE)
+        rownames(result)[10L:12L] <- sub("^[^.]+", "mu",
+          rownames(result)[1L:3L], perl = TRUE)
+        map <- map_grofit_names(opm.fast = TRUE)
+        result <- result[names(map), , drop = FALSE]
+        rownames(result) <- as.character(map)
+        attr(result, OPTIONS) <- options
+      }
+    )
+    attr(result, METHOD) <- method
+  }
 
-  )
-
-  attr(result, PROGRAM) <- program
+  tmp <- opm_string(version = TRUE)
+  attr(result, SOFTWARE) <- tmp[1L]
+  attr(result, VERSION) <- tmp[2L]
 
   if (L(plain))
     return(result)
@@ -311,9 +347,9 @@ setMethod("do_aggr", OPM, function(object, boot = 100L, verbose = FALSE,
 setMethod("do_aggr", "matrix", function(object, by, fun, sep = ".", ...) {
   if (is.atomic(by))
     by <- list(by = by)
-  result <- stats::aggregate(x = object, by = by, FUN = fun, ...,
-    simplify = TRUE)
-  rn <- result[, by.cols <- seq_len(length(by)), drop = FALSE]
+  ## TODO: check for faster alternatives
+  result <- aggregate(x = object, by = by, FUN = fun, ..., simplify = TRUE)
+  rn <- result[, by.cols <- seq_along(by), drop = FALSE]
   rn <- apply(rn, 1L, paste, collapse = sep)
   result <- as.matrix(result[, -by.cols, drop = FALSE])
   rownames(result) <- rn
@@ -321,7 +357,7 @@ setMethod("do_aggr", "matrix", function(object, by, fun, sep = ".", ...) {
 }, sealed = SEALED)
 
 setMethod("do_aggr", "OPMS", function(object, ...) {
-  new(OPMS, plates = lapply(object@plates, FUN = do_aggr, ...))
+  new(OPMS, plates = lapply(X = object@plates, FUN = do_aggr, ...))
 }, sealed = SEALED)
 
 
@@ -335,8 +371,8 @@ setMethod("do_aggr", "OPMS", function(object, ...) {
 #' @param x Object of class \sQuote{boot}.
 #' @param ci Numeric scalar. See \code{\link{fast_estimate}}.
 #' @param as.pe Character scalar. See \code{\link{fast_estimate}}.
-#' @param type Character scalar. See \code{\link{boot.ci}} from the
-#'   \pkg{boot} package.
+#' @param type Character scalar. See \code{\link{boot.ci}} from the \pkg{boot}
+#'   package.
 #' @param fill.nas Logical scalar. Assume that if the CI borders are both
 #'   \code{NA} bootstrapping yielded constant values if the point estimate is
 #'   not \code{NA}, and replace the CI borders by the point estimate in such
@@ -357,20 +393,20 @@ pe_and_ci.boot <- function(x, ci = 0.95, as.pe = c("median", "mean", "pe"),
   LL(ci, fill.nas)
   as.pe <- match.arg(as.pe)
   type <- match.arg(type)
-  if (nrow(x$t) == 0L) {
+  if (nrow(x$t)) {
+    cis <- lapply(seq_along(x$t0), FUN = boot.ci, boot.out = x, conf = ci,
+      type = type, ...)
+    ok <- !vapply(cis, is.null, logical(1L))
+    cis[!ok] <- list(c(NA_real_, NA_real_))
+    cis[ok] <- lapply(cis[ok], `[[`, type, exact = FALSE)
+    cis[ok] <- lapply(cis[ok], FUN = last, i = 2L)
+    cis <- do.call(cbind, cis)
+  } else {
     if (as.pe != "pe") {
       warning("zero bootstrap replicates -- using real point estimate")
       as.pe <- "pe"
     }
     cis <- matrix(nrow = 2L, ncol = length(x$t0), data = NA_real_)
-  } else {
-    cis <- lapply(seq_along(x$t0), FUN = boot::boot.ci, boot.out = x,
-      conf = ci, type = type, ...)
-    ok <- !unlist(lapply(cis, is.null))
-    cis[!ok] <- list(c(NA_real_, NA_real_))
-    cis[ok] <- lapply(cis[ok], `[[`, type, exact = FALSE)
-    cis[ok] <- lapply(cis[ok], FUN = last, i = 2L)
-    cis <- do.call(cbind, cis)
   }
   rownames(cis) <- c("ci.low", "ci.high")
   point.est <- case(as.pe,
@@ -395,22 +431,22 @@ pe_and_ci.boot <- function(x, ci = 0.95, as.pe = c("median", "mean", "pe"),
 #' (maximum height). This is not normally directly called by an \pkg{opm} user
 #' but via \code{\link{do_aggr}}.
 #'
-#' @param x Matrix as output by \code{\link{measurements}}, i.e. with the
-#'   time points in the first columns and the measurements in the remaining
-#'   columns (there must be at least two). For deviations from this scheme see
+#' @param x Matrix as output by \code{\link{measurements}}, i.e. with the time
+#'   points in the first columns and the measurements in the remaining columns
+#'   (there must be at least two). For deviations from this scheme see
 #'   \code{time.pos} and \code{transposed}.
-#' @param what Character scalar. Which parameter to estimate. Currently only
-#'   two are supported.
+#' @param what Character scalar. Which parameter to estimate. Currently only two
+#'   are supported.
 #' @param boot Integer scalar. Number of bootstrap replicates. Note that under
 #'   the default settings for \code{as.pe}, bootstrapping is also necessary to
 #'   obtain the point estimate.
-#' @param ci Confidence interval to use in the output. Ignored if \code{boot}
-#'   is not positive.
+#' @param ci Confidence interval to use in the output. Ignored if \code{boot} is
+#'   not positive.
 #' @param as.pe Character scalar determining what to output as the point
 #'   estimate. Either \sQuote{median}, \sQuote{mean} or \sQuote{pe}; the first
 #'   two calculate the point estimate from the bootstrapping replicates, the
-#'   third one use the real point estimate. If \code{boot} is 0, \code{as.pe}
-#'   is reset to \sQuote{pe}, if necessary, and a warning is issued.
+#'   third one use the real point estimate. If \code{boot} is 0, \code{as.pe} is
+#'   reset to \sQuote{pe}, if necessary, and a warning is issued.
 #' @param ci.type Character scalar determining the way the confidence intervals
 #'   are calculated. Either \sQuote{norm}, \sQuote{basic} or \sQuote{perc}; see
 #'   \code{boot.ci} from the \pkg{boot} package for details.
@@ -425,15 +461,15 @@ pe_and_ci.boot <- function(x, ci = 0.95, as.pe = c("median", "mean", "pe"),
 #'
 #' @export
 #' @return Numeric matrix with three rows (point estimate, lower and upper CI)
-#'   and as many columns as data columns (or rows) in \code{x}. If \code{raw}
-#'   is \code{TRUE}, an object of the class \sQuote{boot}.
+#'   and as many columns as data columns (or rows) in \code{x}. If \code{raw} is
+#'   \code{TRUE}, an object of the class \sQuote{boot}.
 #' @family aggregation-functions
 #' @seealso boot::boot grofit::grofit
 #' @keywords smooth
 #'
 #' @examples
-#' data("vaas_1")
-#' x <- fast_estimate(measurements(vaas_1))
+#' data(vaas_1)
+#' summary(x <- fast_estimate(measurements(vaas_1)))
 #' stopifnot(identical(dim(x), c(3L, 96L)))
 #'
 setGeneric("fast_estimate", function(x, ...) standardGeneric("fast_estimate"))
@@ -456,7 +492,7 @@ setMethod("fast_estimate", "matrix", function(x, what = c("AUC", "A"),
       boot_fun <- function(x, w) colSums(x[w, , drop = FALSE] * y[w])
     }
   )
-  result <- boot::boot(data = x, statistic = boot_fun, R = boot, ...)
+  result <- boot(data = x, statistic = boot_fun, R = boot, ...)
   if (raw)
     return(result)
   result <- pe_and_ci(result, ci = ci, as.pe = as.pe, type = ci.type,
@@ -471,5 +507,3 @@ setMethod("fast_estimate", "matrix", function(x, what = c("AUC", "A"),
 
 
 
-
-
