@@ -13,20 +13,19 @@
 ################################################################################
 
 
-invisible(lapply(c("optparse", "opm", "pkgutils"), library, quietly = TRUE,
+invisible(lapply(c("optparse", "pkgutils", "opm"), library, quietly = TRUE,
   warn.conflicts = FALSE, character.only = TRUE))
 
 
 MD.OUTFILE <- "metadata.csv"
-RESULT.VALUES <- c("clean", "plot", "split", "template", "yaml")
-RESULT.DESCS <- c(
+RESULT <- c(
   "Clean filenames by removing non-word characters except dots and dashes.",
   "Draw plots as postscript files, one per input file.",
   "Split OmniLog(R) CSV files into one file per plate.",
   "Collect a template for adding metadata.",
   "Convert input OmniLog(R) CSV (or opm YAML) files to opm YAML."
 )
-names(RESULT.DESCS) <- RESULT.VALUES
+names(RESULT) <- c("clean", "plot", "split", "template", "yaml")
 
 
 ################################################################################
@@ -52,13 +51,13 @@ run_plot_mode <- function(input, opt) {
   else
     xy_plot
   io_fun <- function(infile, outfile) {
-    x <- read_opm(infile, gen.iii = opt$Gen3)
+    x <- read_opm(infile, gen.iii = opt$type)
     postscript(outfile)
     print(plot_fun(x))
     dev.off()
   }
-  batch_process(names = input, proc = opt$processes, out.ext = "ps", 
-    io.fun = io_fun, outdir = opt$dir, verbose = !opt$quiet, 
+  batch_process(names = input, proc = opt$processes, out.ext = "ps",
+    io.fun = io_fun, outdir = opt$dir, verbose = !opt$quiet,
     overwrite = opt$overwrite, include = opt$include, exclude = opt$exclude)
 }
 
@@ -104,20 +103,24 @@ run_yaml_mode <- function(input, opt) {
   if (!nzchar(opt$dir))
     opt$dir <- NULL
   if (opt$aggregate) {
-    aggr.args <- list(boot = opt$bootstrap, verbose = !opt$quiet, 
+    aggr.args <- list(boot = opt$bootstrap, verbose = !opt$quiet,
       cores = opt$processes)
     if (opt$fast)
-      aggr.args$program <- "opm-fast"
+      aggr.args$method <- "opm-fast"
   } else
     aggr.args <- NULL
+  if (opt$discretize)
+    disc.args <- list(cutoff = opt$weak, plain = FALSE)
+  else
+    disc.args <- NULL
   md.args <- if (is.null(opt$mdfile))
     NULL
   else
     list(md = opt$mdfile, sep = opt$sep, replace = opt$exchange)
-  batch_opm_to_yaml(names = input, proc = proc,
+  batch_opm_to_yaml(names = input, proc = proc, disc.args = disc.args,
     aggr.args = aggr.args, md.args = md.args, outdir = opt$dir,
     verbose = !opt$quiet, overwrite = opt$overwrite, include = opt$include,
-    exclude = opt$exclude, gen.iii = opt$Gen3) 
+    exclude = opt$exclude, gen.iii = opt$type)
 }
 
 
@@ -142,54 +145,77 @@ option.parser <- OptionParser(option_list = list(
   make_option(c("-d", "--dir"), type = "character", default = ".",
     help = "Output directory (empty => input directory) [default: %default]"),
 
-  make_option(c("-e", "--exclude"), type = "character", default = "", 
+  make_option(c("-e", "--exclude"), type = "character", default = "",
     help = "File exclusion globbing pattern [default: <none>]",
     metavar = "PATTERN"),
 
   make_option(c("-f", "--fast"), action = "store_true", default = FALSE,
     help = "When aggregating, use fast method [default: %default]"),
-  
+
   # A bug in Rscript causes '-g' to generate strange warning messages.
   # See https://stat.ethz.ch/pipermail/r-devel/2008-January/047944.html
-  make_option(c("-G", "--Gen3"), action = "store_true", default = FALSE,
-    help = "Change plate type to generation III [default: %default]"),
-  
-  make_option(c("-i", "--include"), type = "character", default = NULL, 
-    help = "File inclusion globbing pattern [default: <see package>]", 
+  #make_option(c("-G", "--Gen3"), action = "store_true", default = FALSE,
+  #  help = "Change plate type to generation III [default: %default]"),
+
+  # h
+
+  make_option(c("-i", "--include"), type = "character", default = NULL,
+    help = "File inclusion globbing pattern [default: <see package>]",
     metavar = "PATTERN"),
 
-  make_option(c("-l", "--level"), action = "store_true", default = FALSE, 
-    help = "When plotting, draw levelplot [default: %default]"),            
-  
+  # j, k
+
+  make_option(c("-l", "--level"), action = "store_true", default = FALSE,
+    help = "When plotting, draw levelplot [default: %default]"),
+
   make_option(c("-m", "--mdfile"), type = "character",
     default = NULL, metavar = "NAME",
     help = "Metadata infile (also used as outfile if given) [default: <none>]"),
 
+  make_option(c("-n", "--encoding"), type = "character",
+    default = "", metavar = "NAME",
+    help = "Assumed character encoding in CSV input [default: %default]"),
+
   make_option(c("-o", "--overwrite"), type = "character", default = "older",
-    help = "Overwrite pre-existing output files [default: %default]", 
+    help = "Overwrite pre-existing output files [default: %default]",
     metavar = "MODE"),
 
   make_option(c("-p", "--processes"), type = "integer", default = 1L,
-    help = paste("Number of processes to spawn (>1 needs 'multicore')", 
-      "[default: %default]"), 
+    help = paste("Number of processes to spawn (>1 needs 'multicore')",
+      "[default: %default]"),
     metavar = "NUMBER"),
-  
+
   make_option(c("-q", "--quiet"), action = "store_true", default = FALSE,
     help = "Do not run verbosely [default: %default]"),
 
   make_option(c("-r", "--result"), type = "character", default = "yaml",
     metavar = "MODE", help = sprintf(
-      "Main result mode; possible values: %s [default: %%default]", 
-      paste(RESULT.VALUES, collapse = ", "))),
+      "Main result mode; possible values: %s [default: %%default]",
+      paste(names(RESULT), collapse = ", "))),
 
   make_option(c("-s", "--sep"), type = "character", default = "\t",
-    help = "Field separator for metadata files [default: <tab>]", 
+    help = "Field separator for metadata files [default: <tab>]",
     metavar = "CHAR"),
-  
+
+  make_option(c("-t", "--type"), type = "character", default = "",
+    help = "Change plate type to this one [default: %default]",
+    metavar = "CHAR"),
+
+  # u, v
+
+  make_option(c("-w", "--weak"), action = "store_true", default = FALSE,
+    help = paste("When discretizing, estimate intermediary (weak) reaction",
+      "[default: %default]")),
+
   make_option(c("-x", "--exchange"), action = "store_true", default = FALSE,
     help = paste("Exchange old by new metadata instead of appending",
-      "[default: %default]"))
-  
+      "[default: %default]")),
+
+  # y
+
+  make_option(c("-z", "--discretize"), action = "store_true", default = FALSE,
+    help = "Discretize after estimating curve parameters [default: %default]")
+
 ))
 
 
@@ -208,18 +234,25 @@ if (is.null(opt$include))
 
 if (length(input) == 0L) {
   print_help(option.parser)
-  message(listing(RESULT.DESCS, header = "The output modes are:", footer = "",
+  message(listing(RESULT, header = "The output modes are:", footer = "",
     prepend = 5L, indent = 10L))
   quit(status = 1L)
 }
 
 
-switch(match.arg(opt$result, RESULT.VALUES),
+invisible(opm_opt(file.encoding = opt$encoding))
+
+
+case(match.arg(opt$result, names(RESULT)),
   clean = run_clean_mode(input, opt),
   plot = run_plot_mode(input, opt),
   split = run_split_mode(input, opt),
   template = run_template_mode(input, opt),
-  yaml = run_yaml_mode(input, opt),
-  stop("unknown 'result' mode: ", opt$result)      
+  yaml = run_yaml_mode(input, opt)
 )
 
+
+################################################################################
+
+
+
