@@ -1,22 +1,25 @@
 
+
 library(testthat)
+context("Testing the IO functions of the opm package")
 
 
-context("Testing the IO functions.")
+if (!exists("TEST.DIR"))
+  attach(objects_for_testing())
 
+# This file does not exist, hence testfile_dir() would not return it
+INFILES.2 <- c(INFILES, sub("Example_3.csv.xz", "Example_4.csv.xz", INFILES[3L],
+  fixed = TRUE))
 
-loc <- source_location()
-if (is.null(loc))
-  loc <- .path.package("opm")
-  
-TEST.DIR <- test_file_dir(loc)
-INFILES <- file.path(TEST.DIR, sprintf("Example_%i.csv.xz", 1L:4L))
 OUTDIR <- tempdir()
-OUTFILES <- paste(file.path(OUTDIR, sub("\\.csv\\.xz$", "", 
-  basename(INFILES))), "txt", sep = ".")
+OUTFILES <- paste(file.path(OUTDIR, sub("\\.csv\\.xz$", "",
+  basename(INFILES.2))), "txt", sep = ".")
 
 
-# A silly function for testing batch_process()
+################################################################################
+
+
+# A silly function for testing batch_process() etc.
 #
 copy_head <- function(infile, outfile) {
   data <- readLines(infile, n = 5L)
@@ -24,25 +27,46 @@ copy_head <- function(infile, outfile) {
 }
 
 
+# Tests whether two character vectors refer to the same file path (are
+# identical once normalized); standardize the path separators (to UNIX style).
+#
+expect_path_equivalent <- function(actual, expected) {
+  norm_path <- function(x) normalizePath(x, winslash = "/", mustWork = FALSE)
+  expect_equal(norm_path(actual), norm_path(expected))
+}
+
+
+# Tests whether two character vectors are identical except for differences
+# in the path separators (Windows vs. UNIX style).
+#
+expect_path_equal <- function(actual, expected) {
+  clean <- function(x) chartr("\\", "/", x)
+  expect_equal(clean(actual), clean(expected))
+}
+
+
 ################################################################################
 #
 # IO helpers
 #
 
-
+## file_pattern
 test_that("file patterns can be constructed", {
-  
   default.pat <- "\\.(csv|ya?ml)(\\.(bz2|gz|lzma|xz))?$"
   expect_equal(default.pat, file_pattern())
   expect_equal("\\.csv$", file_pattern(type = "csv", compressed = FALSE))
-  
+})
+
+
+##  extended_file_pattern
+test_that("extended file patterns can be constructed", {
+  default.pat <- "\\.(csv|ya?ml)(\\.(bz2|gz|lzma|xz))?$"
   expect_equal(NULL, extended_file_pattern(NULL))
   expect_equal(default.pat, extended_file_pattern(list()))
   expect_error(extended_file_pattern("*.csv"))
   expect_equal("*.csv", extended_file_pattern("*.csv", wildcard = FALSE))
   expect_equal("^.*\\.csv$", extended_file_pattern("*.csv", wildcard = TRUE))
   expect_equal("5", extended_file_pattern(5, wildcard = FALSE))
-  
 })
 
 
@@ -52,18 +76,44 @@ test_that("file patterns can be constructed", {
 #
 
 
-test_that("example file 1 in old style can be read", {
-  example.file.old.style.1 <- file.path(TEST.DIR,
-    "Example_Old_Style_1.csv.xz")
-  opm.2 <- read_single_opm(example.file.old.style.1)
-  expect_is(opm.2, "OPM")
-  expect_equal(filename(opm.2), example.file.old.style.1)
-  expect_equal(plate_type(opm.2), "PM20")
-  expect_equal(setup_time(opm.2), "Apr 11 2011 5:08 PM")
-  expect_equal(position(opm.2), "12-A")
-  expect_equal(hours(opm.2), 91.25)
-  expect_equal(dim(opm.2), c(366, 96))
-  expect_equal(metadata(opm.2), list())
+## repair_oth
+## UNTESTED
+
+## read_old_opm
+## UNTESTED
+
+## read_new_opm
+## UNTESTED
+
+## read_microstation_opm
+## UNTESTED
+
+## read_opm_yaml
+## UNTESTED
+
+## read_single_opm
+test_that("the example file in old style can be read", {
+  x <- read_single_opm(FILE.OLD.STYLE)
+  expect_is(x, "OPM")
+  expect_equal(filename(x), FILE.OLD.STYLE)
+  expect_equal(plate_type(x), "PM20")
+  expect_equal(setup_time(x), "Apr 11 2011 5:08 PM")
+  expect_equal(position(x), "12-A")
+  expect_equal(hours(x), 91.25)
+  expect_equal(dim(x), c(366, 96))
+  expect_equal(metadata(x), list())
+})
+
+## read_single_opm
+test_that("the ID-run example file can be read", {
+  x <- read_single_opm(FILE.ID.RUN)
+  expect_is(x, "OPM")
+  expect_equal(filename(x), FILE.ID.RUN)
+  expect_equal(plate_type(x), SPECIAL_PLATES[["gen.iii"]])
+  expect_equal(setup_time(x), "2/28/2012 9:59:53 AM")
+  expect_equal(position(x), " 9-B")
+  expect_equal(dim(x), c(1, 96))
+  expect_equal(metadata(x), list())
 })
 
 
@@ -72,21 +122,107 @@ test_that("example file 1 in old style can be read", {
 # Input of multiple OPM files
 #
 
-    
+
+## read_opm
+test_that("the ECO-run example file can be read", {
+  example.file.eco <- file.path(TEST.DIR, "Example_Ecoplate.csv.xz")
+  x <- read_opm(example.file.eco)
+  expect_is(x, "OPMS")
+  expect_equal(unique(filename(x)), example.file.eco)
+  expect_equal(plate_type(x), SPECIAL_PLATES[["eco"]])
+  expect_equal(dim(x), c(11, 1, 96))
+  md.len <- unique(vapply(metadata(x), length, integer(1L)))
+  expect_equal(md.len, 106L)
+})
+
+
+## read_opm
+test_that("read_opm can read a single file", {
+  files <- INFILES.2[1L]
+  opm.1 <- read_opm(files)
+  expect_is(opm.1, "OPM")
+  opm.1 <- read_opm(files, convert = "try")
+  expect_is(opm.1, "OPM")
+  opm.1 <- read_opm(files, convert = "yes")
+  expect_is(opm.1, "OPM")
+  opm.1 <- read_opm(files, convert = "no")
+  expect_is(opm.1, "list")
+  expect_equal(1L, length(opm.1))
+})
+
+## read_opm
+test_that("read_opm can read two compatible files", {
+  files <- INFILES.2[1L:2L]
+  opm.1 <- read_opm(files)
+  expect_is(opm.1, "OPMS")
+  opm.1 <- read_opm(files, convert = "try")
+  expect_is(opm.1, "OPMS")
+  expect_equal(NULL, names(plates(opm.1)))
+  opm.1 <- read_opm(files, convert = "yes")
+  expect_is(opm.1, "OPMS")
+  expect_equal(NULL, names(plates(opm.1)))
+  opm.1 <- read_opm(files, convert = "no")
+  expect_is(opm.1, "list")
+  expect_equal(2L, length(opm.1))
+  expect_equal(NULL, names(opm.1))
+})
+
+## read_opm
+test_that("read_opm can read three partially incompatible files", {
+
+  files <- testfile_dir(files = c("Example_1.csv.xz", "Example_2.csv.xz",
+    "Example_Old_Style_1.csv.xz"))
+
+  expect_warning(opm.1 <- read_opm(files))
+  expect_is(opm.1, "list")
+  expect_equal(3L, length(opm.1))
+  expect_equal(NULL, names(opm.1))
+
+  expect_warning(opm.1 <- read_opm(files, convert = "try"))
+  expect_is(opm.1, "list")
+  expect_equal(3L, length(opm.1))
+  expect_equal(NULL, names(opm.1))
+
+  expect_error(opm.1 <- read_opm(files, convert = "yes"))
+
+  opm.1 <- read_opm(files, convert = "no")
+  expect_is(opm.1, "list")
+  expect_equal(3L, length(opm.1))
+  expect_equal(NULL, names(opm.1))
+
+  opm.1 <- read_opm(files, convert = "sep")
+  expect_is(opm.1, "list")
+  expect_equal(2L, length(opm.1))
+  expect_true(all(sapply(opm.1, class) == "list"))
+  expect_true(all(sapply(opm.1[[1]], class) == OPM))
+  expect_true(all(sapply(opm.1[[2]], class) == OPM))
+
+  opm.1 <- read_opm(files, convert = "grp")
+  expect_is(opm.1, "list")
+  expect_equal(2L, length(opm.1))
+  expect_is(opm.1[[1L]], OPMS)
+  expect_is(opm.1[[2L]], OPM)
+
+})
+
+
+## explode_dir
 test_that("explode_dir finds the files it should find", {
   files <- explode_dir(TEST.DIR, include = file_pattern(type = "csv"),
-    wildcard = FALSE)  
-  expect_true(all(grepl(TEST.DIR, files)))
-  expect_equal(length(files), 8L)
+    wildcard = FALSE)
+  # TODO: might be safer to apply normalizePath() here, too
+  expect_true(all(grepl(TEST.DIR, files, fixed = TRUE)))
+  expect_equal(length(files), 9L)
   expect_equal(names(files), NULL)
   files <- explode_dir(TEST.DIR, include = file_pattern(type = "csv"),
-    exclude = "old", wildcard = FALSE)  
-  expect_true(all(grepl(TEST.DIR, files)))
-  expect_equal(length(files), 3L)
+    exclude = "old", wildcard = FALSE)
+  # TODO: might be safer to apply normalizePath() here, too
+  expect_true(all(grepl(TEST.DIR, files, fixed = TRUE)))
+  expect_equal(length(files), 5L)
   expect_equal(names(files), NULL)
 })
-  
 
+## explode_dir
 test_that("explode_dir uses list pattern input", {
   csv.files <- explode_dir(TEST.DIR, include = file_pattern(type = "csv"),
     wildcard = FALSE)
@@ -98,8 +234,8 @@ test_that("explode_dir uses list pattern input", {
     exclude = "old", wildcard = FALSE)
   expect_equal(files, old.csv.files)
 })
-  
 
+## explode_dir
 test_that("explode_dir uses globbing patterns", {
   csv.files <- explode_dir(TEST.DIR, include = file_pattern(type = "csv"),
     wildcard = FALSE)
@@ -111,8 +247,8 @@ test_that("explode_dir uses globbing patterns", {
     exclude = "*old*")
   expect_equal(files, old.csv.files)
 })
-  
 
+## explode_dir
 test_that("explode_dir uses regex patterns", {
   csv.files <- explode_dir(TEST.DIR, include = file_pattern(type = "csv"),
     wildcard = FALSE)
@@ -121,28 +257,28 @@ test_that("explode_dir uses regex patterns", {
   files <- explode_dir(TEST.DIR, include = ".*\\.csv\\.xz$",
     wildcard = FALSE)
   expect_equal(files, csv.files)
-  files <- explode_dir(TEST.DIR, include = ".*\\.csv\\.xz$", 
+  files <- explode_dir(TEST.DIR, include = ".*\\.csv\\.xz$",
     wildcard = FALSE, exclude = ".*old.*")
   expect_equal(files, old.csv.files)
 })
-  
 
+## explode_dir
 test_that("explode_dir deals with non-existing files", {
   x <- c("0123456789", TEST.DIR)
   expect_error(explode_dir(x))
   expect_warning(explode_dir(x, missing.error = FALSE))
 })
 
-    
-## UNTESTED: opm_files() read_opm() [see test_opm.R and test_opms.R]
-    
+## opm_files
+## UNTESTED
+
 
 ################################################################################
 #
 # Metadata IO
 #
 
-
+## to_metadata
 test_that("to_metadata converts objects in the right way", {
   x <- data.frame(a = 1:10, b = letters[1:10])
   expect_equivalent(c("integer", "factor"), sapply(x, class))
@@ -162,9 +298,9 @@ test_that("to_metadata converts objects in the right way", {
 # Batch-collection functions
 #
 
-
+## batch_collect
 test_that("batch collection works as expected", {
-  files <- INFILES[1L:2L]
+  files <- INFILES.2[1L:2L]
   got <- batch_collect(files, readLines, fun.arg = list(n = 5L))
   expect_is(got, "list")
   expect_equal(files, names(got))
@@ -172,10 +308,10 @@ test_that("batch collection works as expected", {
   expect_true(all(sapply(got, length) == 5L))
   expect_that(got <- batch_collect(files, readLines, fun.arg = list(n = 5L),
     demo = TRUE), shows_message())
-  expect_equal(got, files)
+  expect_path_equal(got, files)
   expect_that(got <- batch_collect(TEST.DIR, include = '*.csv.xz',
-     readLines, fun.arg = list(n = 5L), demo = TRUE), shows_message())
-  expect_equal(got[1L:2L], files)
+    readLines, fun.arg = list(n = 5L), demo = TRUE), shows_message())
+  expect_path_equal(got[1L:2L], files)
   expect_is(got <- batch_collect(files, readLines, fun.arg = list(n = 5L),
     exclude = "*.csv.xz", wildcard = TRUE), "list")
   expect_equal(length(got), 0L)
@@ -184,34 +320,43 @@ test_that("batch collection works as expected", {
   expect_equal(character(), got)
 })
 
-
+## collect_template
 test_that("templates can be collected", {
-  
-  files <- INFILES[1L:3L]
-  
-  template <- collect_template(files, exclude = "*3*")
+
+  files <- INFILES.2[1L:3L]
+
+  expect_equal(length(files), 3L)
+  expect_true(all(file.exists(files)))
+
+  # if 'exclude' was not specific enough, 'template' would be NULL
+  template <- collect_template(files, exclude = "*Example*_3.*")
   expect_is(template, "data.frame")
   expect_equal(colnames(template), c("Setup Time", "Position", "File"))
   expect_equal(nrow(template), 2L)
-  expect_true(all(! is.na(template)))
-  expect_true(all("character" == sapply(template, class)))
-  
-  expect_that(template <- collect_template(files, exclude = "*3*", demo = TRUE),
-    shows_message())
+  expect_true(all(!is.na(template)))
+  expect_true(all(vapply(template, inherits, logical(1L), "character")))
+
+  expect_that(template <- collect_template(files, exclude = "*Example*_3.*",
+    demo = TRUE), shows_message())
+  # if 'exclude' was not specific enough, 'template' would be empty
   expect_equal(template, files[1L:2L])
 
 })
 
-
+## collect_template
 test_that("templates can be collected and written to files", {
-  
-  files <- INFILES[1L:3L]
+
+  files <- INFILES.2[1L:3L]
   outfile <- tempfile()
   infile <- tempfile()
-  
+
+  expect_equal(length(files), 3L)
+  expect_true(all(file.exists(files)))
   expect_false(any(file.exists(c(outfile, infile))))
-  
-  template <- collect_template(files, exclude = "*3*", outfile = outfile)
+
+  # if 'exclude' was not specific enough, 'template' would be NULL
+  template <- collect_template(files, exclude = "*Example*_3.*",
+    outfile = outfile)
   expect_true(file.exists(outfile))
   expect_false(file.exists(infile))
   expect_is(template, "data.frame")
@@ -220,31 +365,38 @@ test_that("templates can be collected and written to files", {
   expect_true(all(! is.na(template)))
   expect_true(all("character" == sapply(template, class)))
   unlink(outfile)
-  
-  expect_error(template <- collect_template(files, exclude = "*3*", 
+
+  expect_error(template <- collect_template(files, exclude = "*Example*_3.*",
     outfile = outfile, previous = infile))
   expect_false(file.exists(outfile))
   expect_false(file.exists(infile))
 
   unlink(c(infile, outfile))
-  
+
 })
 
-
+## collect_template
 test_that("templates can be collected with added columns", {
-  
-  files <- INFILES[1L:3L]
+
+  files <- INFILES.2[1L:3L]
   to.add <- c("A", "B")
-  
-  template <- collect_template(files, exclude = "*3*", add.cols = to.add)
+
+  expect_equal(length(files), 3L)
+  expect_true(all(file.exists(files)))
+
+  # if 'exclude' was not specific enough, 'template' would be NULL
+  template <- collect_template(files, exclude = "*Example*_3.*",
+    add.cols = to.add)
   expect_is(template, "data.frame")
-  expect_equal(colnames(template), c("Setup Time", "Position", "File", to.add))
+  expect_equal(colnames(template),
+    c("Setup Time", "Position", "File", to.add))
   expect_equal(nrow(template), 2L)
   expect_true(all(!is.na(template[, 1L:3L])))
   expect_true(all(is.na(template[, to.add])))
   expect_true(all("character" == sapply(template, class)))
 
-  expect_that(template <- collect_template(files, exclude = "*3*", 
+  # if 'exclude' was not specific enough, 'template' would be empty
+  expect_that(template <- collect_template(files, exclude = "*Example*_3.*",
     add.cols = to.add, demo = TRUE), shows_message())
   expect_equal(template, files[1L:2L])
 
@@ -257,15 +409,19 @@ test_that("templates can be collected with added columns", {
 #
 
 
+## process_io
+## UNTESTED
+
+## batch_process
 test_that("batch conversion works in demo mode", {
-  
-  infiles <- INFILES
+
+  infiles <- INFILES.2
   expect_false(any(file.exists(OUTFILES)))
-  
+
   # Demo run not allowing missing input files
-  expect_error(got <- batch_process(infiles, out.ext = 'txt',            
+  expect_error(got <- batch_process(infiles, out.ext = 'txt',
     io.fun = copy_head, outdir = OUTDIR, verbose = FALSE, demo = TRUE))
-  
+
   # Demo run
   expect_that(got <- batch_process(infiles, out.ext = 'txt', io.fun = copy_head,
     outdir = OUTDIR, missing.error = FALSE, verbose = TRUE, demo = TRUE),
@@ -273,17 +429,18 @@ test_that("batch conversion works in demo mode", {
   expect_is(got, "matrix")
   expect_equal(got[, 1L], infiles[-4L])
   expect_equal(got[, 2L], OUTFILES[-4L])
-  
+
   expect_false(any(file.exists(OUTFILES)))
-  
-})  
-  
-  
+
+})
+
+
+## batch_process
 test_that("batch conversion works", {
-  
-  infiles <- INFILES
+
+  infiles <- INFILES.2
   expect_false(any(file.exists(OUTFILES)))
-  
+
   # Real run with forced overwriting
   for (i in 1L:2L) {
     expect_warning(got <- batch_process(infiles, out.ext = 'txt',
@@ -324,10 +481,10 @@ test_that("batch conversion works", {
   expect_equal(OUTFILES[-4L], got[, "outfile"])
   expect_true(all(got[, "before"] == "outfile not empty and newer"))
   expect_true(all(got[, "after"] == ""))
-  
+
   # Clean up
   unlink(OUTFILES)
-  
+
 })
 
 
@@ -336,15 +493,15 @@ test_that("batch conversion works", {
 # Batch IO with OPM objects
 #
 
-
+## batch_opm_to_yaml
 test_that("batch conversion to yaml works", {
-  infiles <- INFILES
+  infiles <- INFILES.2
   expect_warning(got <- batch_opm_to_yaml(infiles, missing.error = FALSE,
     demo = TRUE))
   infiles <- infiles[-4L]
   expect_is(got, "matrix")
   expect_equal(got[, 1L], infiles)
-  expect_equal(got[, 2L], sub("\\.csv\\.xz$", ".yml", infiles))
+  expect_path_equal(got[, 2L], sub("\\.csv\\.xz$", ".yml", infiles))
   infiles <- infiles[1L]
   outdir <- tempdir()
   exp.outfile <- file.path(outdir, "Example_1.yml")
@@ -360,9 +517,9 @@ test_that("batch conversion to yaml works", {
 # Splitting files
 #
 
+## split_files
+test_that("files can be split", {
 
-test_that("files can be split", { # see also the example
-  
   tmp <- c(tempfile(), tempfile())
   # Dummy FASTA files
   x <- c(">Ahoernchen", "acataggacaggataggacaattagatacagat", "acggat",
@@ -373,27 +530,27 @@ test_that("files can be split", { # see also the example
   write(x, tmp[1L])
   write(y, tmp[2L])
 
-  expect_that(got <- split_files(tmp, ">*", wildcard = TRUE, demo = TRUE), 
+  expect_that(got <- split_files(tmp, ">*", wildcard = TRUE, demo = TRUE),
     shows_message())
   expect_is(got, "list")
   expect_equal(names(got), tmp)
-  
+
   got <- split_files(tmp, ">*", wildcard = TRUE)
   expect_is(got, "list")
   expect_equal(names(got), tmp)
-  
+
   got.1 <- got[[1L]]
   expect_equal(length(got.1), 2L)
-  expect_true(all(file.exists(got.1)))  
+  expect_true(all(file.exists(got.1)))
   unlink(got.1)
 
   got.2 <- got[[2L]]
   expect_equal(length(got.2), 3L)
-  expect_true(all(file.exists(got.2)))  
+  expect_true(all(file.exists(got.2)))
   unlink(got.2)
 
   unlink(tmp)
-  
+
 })
 
 
@@ -402,15 +559,16 @@ test_that("files can be split", { # see also the example
 # File renaming
 #
 
-
+## clean_filenames
 test_that("file names can be cleaned", {
-  x <- c("a b/c/x-y-z.txt", "d--z/z?-z. .csv", "/xxx/y y/?_?-abcd*+.txt", 
+  x <- c("a b/c/x-y-z.txt", "d--z/z?-z. .csv", "/xxx/y y/?_?-abcd*+.txt",
     "", "sapif89asdh_&#-*_asdfhu.---", "!$%&+()=?`")
   expect_that(got <- clean_filenames(x, demo = TRUE), shows_message())
   expect_equal(names(got), x[-1L])
   expect_equivalent(got, c("d--z/z-z.csv", "/xxx/y y/abcd.txt",
-    "./__EMPTY__00001__", "./sapif89asdh-asdfhu", "./__EMPTY__00002__"))             
+    "./__EMPTY__00001__", "./sapif89asdh-asdfhu", "./__EMPTY__00002__"))
 })
 
 
+################################################################################
 
